@@ -73,27 +73,6 @@ class TrackerServiceHttpRequestHandler(HttpRequestHandler):
             is_battery_plugged,
             location)
 
-    @http_request(r'^/tracker/activation/request$',
-                  http_method=HttpMethod.POST,
-                  authentication_required=True,
-                  signature_required=True)
-    def request_activation_code(self, request):
-        activation_code_duration = request.get_argument(
-            'duration',
-            data_type=HttpRequest.ArgumentDataType.integer,
-            is_required=False)
-
-        team_id = request.get_argument(
-            'team_id',
-            data_type=HttpRequest.ArgumentDataType.uuid,
-            is_required=True)
-
-        return TrackerService().request_activation_code(
-            request.app_id,
-            request.account_id,
-            team_id,
-            activation_code_duration=activation_code_duration)
-
     @http_request(r'^/tracker/(device_id)/battery/event$',
                   http_method=HttpMethod.POST,
                   authentication_required=False,
@@ -120,6 +99,54 @@ class TrackerServiceHttpRequestHandler(HttpRequestHandler):
             account_id=request.session and request.session.account_id,
             seed=seed)
 
+    @http_request(r'^/tracker/(device_id)/keepalive$',
+                  http_method=HttpMethod.POST,
+                  authentication_required=False,
+                  signature_required=True)
+    def report_keepalive(self, request, device_id):
+        seed = request.get_argument(
+            's',
+            data_type=HttpRequest.ArgumentDataType.string,
+            argument_passing=HttpRequest.ArgumentPassing.query_string,
+            is_required=True)
+
+        battery_level = request.get_argument(
+            'battery_level',
+            data_type=HttpRequest.ArgumentDataType.decimal,
+            is_required=False)
+
+        is_battery_plugged = request.get_argument(
+            'is_battery_plugged',
+            data_type=HttpRequest.ArgumentDataType.boolean,
+            is_required=False)
+
+        location = request.get_argument(
+            'location',
+            data_type=HttpRequest.ArgumentDataType.object,
+            object_class=GeoPoint,
+            is_required=False)
+
+        network_type = request.get_argument(
+            'network_type',
+            data_type=HttpRequest.ArgumentDataType.string,
+            is_required=False)
+
+        mobile_exchanged_byte_count = request.get_argument(
+            'ex_bytes',
+            data_type=HttpRequest.ArgumentDataType.integer,
+            is_required=False)
+
+        return TrackerService().report_keepalive(
+            request.app_id,
+            device_id,
+            seed,
+            account_id=request.session and request.session.account_id,
+            battery_level=battery_level,
+            is_battery_plugged=is_battery_plugged,
+            location=location,
+            mobile_exchanged_byte_count=mobile_exchanged_byte_count,
+            network_type=network_type)
+
     @http_request(r'^/tracker/(device_id)/location$',
                   http_method=HttpMethod.POST,
                   authentication_required=False,
@@ -153,11 +180,46 @@ class TrackerServiceHttpRequestHandler(HttpRequestHandler):
             seed=seed,
             team_id=team_id)
 
+    @http_request(r'^/tracker/activation/request$',
+                  http_method=HttpMethod.POST,
+                  authentication_required=True,
+                  signature_required=True)
+    def request_activation_code(self, request):
+        activation_code_duration = request.get_argument(
+            'duration',
+            data_type=HttpRequest.ArgumentDataType.integer,
+            is_required=False)
+
+        team_id = request.get_argument(
+            'team_id',
+            data_type=HttpRequest.ArgumentDataType.uuid,
+            is_required=True)
+
+        return TrackerService().request_activation_code(
+            request.app_id,
+            request.account_id,
+            team_id,
+            activation_code_duration=activation_code_duration)
+
     @http_request(r'^/tracker/(device_id)/handshake$',
                   http_method=HttpMethod.POST,
                   authentication_required=False,
                   signature_required=True)
     def shake_hands(self, request, device_id):
+        # @TODO: REMOVE THIS PATCH ONCE THE BUG FOUND AND FIXED!
+        user_agent = request.get_header('User-Agent', is_required=False)
+        with open('/home/devops/user_agents.txt', 'at') as fd:
+            fd.write(user_agent + '\n')
+        if request.client_application is None:
+            from majormode.perseus.model.app import ClientApplication
+            from majormode.perseus.model.version import Version
+            request.client_application = ClientApplication(
+                user_agent,
+                Version(1, 0, 0),
+                'Android',
+                Version(1, 0, 0),
+                'FooBar')
+
         # Check whether the information of the agent application has been
         # correctly passed.
         if not request.client_application:
@@ -190,6 +252,11 @@ class TrackerServiceHttpRequestHandler(HttpRequestHandler):
             data_type=HttpRequest.ArgumentDataType.string,
             is_required=True)
 
+        print(f'battery_level: {battery_level}')
+        print(f'is_battery_plugged: {is_battery_plugged}')
+        print(f'is_battery_present: {is_battery_present}')
+        print(f'mac_address: {mac_address}')
+
         return TrackerService().shake_hands(
             request.app_id,
             device_id,
@@ -254,35 +321,7 @@ class TrackerServiceHttpRequestHandler(HttpRequestHandler):
     #
     #
 
-    #
-    # @http_request(r'^/device/(device_id)/keepalive_message$',
-    #               http_method=HttpRequest.HttpMethod.POST,
-    #               authentication_required=False,
-    #               signature_required=True)
-    # def on_device_keepalive_message(self, request, device_id):
-    #     seed = request.get_argument('s',
-    #             data_type=HttpRequest.ArgumentDataType.string,
-    #             argument_passing=HttpRequest.ArgumentPassing.query_string,
-    #             is_required=True)
-    #
-    #     battery_level = request.get_argument('battery_level',
-    #             data_type=HttpRequest.ArgumentDataType.decimal,
-    #             is_required=False)
-    #
-    #     is_battery_plugged = request.get_argument('is_battery_plugged',
-    #             data_type=HttpRequest.ArgumentDataType.boolean,
-    #             is_required=False)
-    #
-    #     location = request.get_argument('location',
-    #             data_type=HttpRequest.ArgumentDataType.object,
-    #             object_class=GeoPoint,
-    #             is_required=False)
-    #
-    #     return XeberusService().on_device_keepalive_message(request.app_id, device_id, seed,
-    #             battery_level=battery_level,
-    #             is_battery_plugged=is_battery_plugged,
-    #             location=location)
-    #
+
     #
     #
     #
